/**
 * @fileoverview To show error when followed bad practices inside mapStateToProps of react redux applications
 * @author Sivanesh
 */
'use strict';

//------------------------------------------------------------------------------
// Rule Definition
//------------------------------------------------------------------------------

module.exports = {
  meta: {
    docs: {
      description:
        'To show error when followed bad practices inside mapStateToProps of react redux applications',
      category: 'Performance',
      recommended: true
    },
    fixable: null, // or "code" or "whitespace"
    schema: [
      // fill in your schema
    ]
  },

  create: function(context) {
    const vulnerableQuery = ':matches(ArrayExpression, ObjectExpression)';
    const idNameQuery = '[id.name="mapStateToProps"]';
    const connectCallQuery = 'CallExpression[callee.name="connect"]';

    //----------------------------------------------------------------------
    // Helpers
    //----------------------------------------------------------------------

    function foundInitialization(node) {
      if (node.parent.type === 'ReturnStatement') return;
      if (
        node.type === 'ObjectExpression' &&
        node.parent.type === 'ArrowFunctionExpression' &&
        node.parent &&
        node.parent.parent &&
        node.parent.parent.id &&
        node.parent.parent.id.name === 'mapStateToProps'
      ) {
        return;
      }
      if (node.parent.type === 'CallExpression') {
        return;
      }

      if (
        node.parent.type === 'LogicalExpression' &&
        node.parent.parent &&
        node.parent.parent.id &&
        node.parent.parent.id.type === 'ObjectPattern'
      ) {
        return;
      }
      if (
        node.parent.type === 'ConditionalExpression' &&
        node.parent.parent &&
        node.parent.parent.id &&
        node.parent.parent.id.type === 'ObjectPattern'
      ) {
        return;
      }

      context.report({
        node,
        message:
          'Causes performance issue as it creates a new reference each time when redux calls this connect.'
      });
    }
    // add a function that takes a list of files it obtains mapStateToProps and count.

    //----------------------------------------------------------------------
    // Public
    //----------------------------------------------------------------------

    return {
      [`FunctionDeclaration${idNameQuery} ${vulnerableQuery}`]: foundInitialization,
      [`VariableDeclarator${idNameQuery} ArrowFunctionExpression ${vulnerableQuery}`]: foundInitialization,
      [`${connectCallQuery} ArrowFunctionExpression ${vulnerableQuery}`]: foundInitialization,
      [`${connectCallQuery} FunctionExpression ${vulnerableQuery}`]: foundInitialization
    };
  }
};
