/**
 * @fileoverview To show error when followed bad practices inside mapStateToProps of react redux applications
 * @author Sivanesh
 */
"use strict";

//------------------------------------------------------------------------------
// Requirements
//------------------------------------------------------------------------------

var rule = require("../../../lib/rules/no-ref-mapstatetoprops"),
  RuleTester = require("eslint").RuleTester;

//------------------------------------------------------------------------------
// Tests
//------------------------------------------------------------------------------

var ruleTester = new RuleTester({ parserOptions: { ecmaVersion: 2018 } });
const ERROR_MESSAGE =
  "Causes performance issue as it creates a new reference each time when redux calls this connect.";
ruleTester.run("no-ref-mapstatetoprops", rule, {
  valid: [
    "function mapStateToProps(state) {return {id: state.agent.id}}",
    "const mapStateToProps = state => {return {arr: state.arr}}",
    "connect(function(state) {return {id: state.module.id, name: state.name}})",
    "connect(state => {return {arr: state.arr}})"
  ],
  invalid: [
    {
      code: "function mapStateToProps(state) {return {mode: []}}",
      errors: [
        {
          message: ERROR_MESSAGE,
          type: "ArrayExpression"
        }
      ]
    },
    {
      code: "connect((state) => { return { mode: state.mode || {} } })",
      errors: [
        {
          message: ERROR_MESSAGE,
          type: "ObjectExpression"
        }
      ]
    },
    {
      code: "function mapStateToProps(state) {return {mode: state.mode || {}}}",
      errors: [
        {
          message: ERROR_MESSAGE,
          type: "ObjectExpression"
        }
      ]
    },
    {
      code:
        "function mapStateToProps(state) { let { preference = {} } = state.userObj; return {mode: state.mode}}",
      errors: [
        {
          message: ERROR_MESSAGE,
          type: "ObjectExpression"
        }
      ]
    }
  ]
});
